#include <stdint.h>
#include <stdio.h>
#include <tgmath.h>
#include <timers.h>
#include <timing.h>

#include "devices.h"
#include "instruments.h"
#include "drv_instrument.h"
#include "drv_dac.h"

#define PI 3.14159
#define DAC_CHAN_X 0
#define DAC_CHAN_Y 1

dac_t *dac;
instrument_t *ctlpanel;

struct FGen {
    uint8_t lastx, lasty;
    uint8_t gen1x,gen1y,gen2x,gen2y,gen3x,gen3y;
    uint8_t mul1,mul2,mul3;
    uint32_t tim;
} fgen;

void genloop(struct FGen fg) {
    uint8_t xv, yv;
    bool upd;
    fg.tim++;
    double phi;
    /* Generator 1 - sawtooth */
    fg.gen1x = (fg.gen1x+1) % 256;
    fg.gen1y = (fg.gen1y+3) % 256;
    /* Generator 2 - Sine / Cosine */
    phi = (fg.tim%1000*2*PI)/1000;
    fg.gen2x = 128*(1.0+(sin(phi)));
    fg.gen2y = 128*(1.0+(cos((phi)));
    /* Generator 3 is manual (no code here) */
    /* now comes the mixer */
    xv = (fg.mul1*fg.gen1x) + (fg.mul2*fg.gen2x) + (fg.mul3*fg.gen3x);
    yv = (fg.mul1*fg.gen1y) + (fg.mul2*fg.gen2y) + (fg.mul3*fg.gen3y);
    upd = false;
    if ((xv!=fg.lastx) || (yv!=fg.lasty)) {
         upd = true;
         fg.lastx = xv;
         fg.lasty = yv;
    }
    if (upd) {
       /* printf("X=%d Y=%d\n",xv,yv); */
       instrument_set_value(ctlpanel,CTLPANEL_XOUT,xv);
       instrument_set_value(ctlpanel,CTLPANEL_YOUT,yv);
       dac_write(dac,DAC_CHAN_X,xv,false);
       dac_write(dac,DAC_CHAN_Y,yv,true);
    }
}

void init() {
    dac = dac_open(DAC_1);
    ctlpanel = instrument_open(CTLPANEL);
    fgen.mul1 = 0;
    fgen.mul2 = 0;
    fgen.mul3 = 1;
    fgen.tim = 0;
    timer_register_handler(&fgen,1000,(timer_handler_t) genloop);
}

void main() {
    uint8_t sel_saw,sel_sine,sel_man;
    init();
    uint64_t fhz = freq_hz();
    printf("NOS031004 started, clock frequency is %d Hz\n",freq_hz());
    for (;;) {
        fgen.gen3x = instrument_get_value(ctlpanel,CTLPANEL_XIN);
        fgen.gen3y = instrument_get_value(ctlpanel,CTLPANEL_YIN);
        sel_saw = instrument_get_value(ctlpanel,CTLPANEL_SEL_SAW);
        sel_sine = instrument_get_value(ctlpanel,CTLPANEL_SEL_SINE);
        sel_man = instrument_get_value(ctlpanel,CTLPANEL_SEL_MAN);
        fgen.mul1 = (sel_saw!=0)?1:0;
        fgen.mul2 = (sel_sine!=0)?1:0;
        fgen.mul3 = (sel_man!=0)?1:0;
    }
}

